/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.applicationprocessor.processor;

import java.util.List;

import org.tinygroup.appconfig.AppConfig;
import org.tinygroup.appconfig.util.AppConfigUtil;
import org.tinygroup.application.Application;
import org.tinygroup.application.ApplicationProcessor;
import org.tinygroup.fileresolver.FileProcessor;
import org.tinygroup.fileresolver.FileResolver;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.parser.filter.NameFilter;
import org.tinygroup.xmlparser.node.XmlNode;

/**
 * 加载处理器，并执行加载
 * 
 * @author luoguo
 * 
 */
public class FileResolverProcessor implements ApplicationProcessor {
	private static final String FILE_RESOLVER_NODE_NAME = "file-resolver-configuration";
	private static Logger logger = LoggerFactory
			.getLogger(FileResolverProcessor.class);
	private XmlNode fileResolverConfiguration;

	private FileResolver fileResolver;

	public FileResolver getFileResolver() {
		return fileResolver;
	}

	public void setFileResolver(FileResolver fileResolver) {
		this.fileResolver = fileResolver;
	}

	public void setApplication(Application application) {

	}

	public void setConfiguration(XmlNode fileResolverNodes) {
		this.fileResolverConfiguration = fileResolverNodes;
	}

	public void start() {
		if (fileResolverConfiguration != null
				&& fileResolverConfiguration.getContent() != null) {
			loadClassPath();
			configFileResolvers();
		} else {
			logger.logMessage(LogLevel.INFO, "没有文件搜索配置信息");
		}
		fileResolver.resolve();
	}

	private void loadClassPath() {
		setResolverClassPath();
		setResolverOrder();
		String threadCount = fileResolverConfiguration
				.getAttribute("thread-count");
		fileResolver.setFileProcessorThreadNum(getThreadNum(threadCount));
		NameFilter<XmlNode> nameFilter = new NameFilter<XmlNode>(
				fileResolverConfiguration);
		List<XmlNode> classPaths = nameFilter.findNodeList("class-path");
		for (XmlNode classPath : classPaths) {
			String path = classPath.getAttribute("path");
			if (path != null && path.length() > 0) {
				logger.logMessage(LogLevel.INFO, "添加手工配置classpath: [{0}]...",
						path);
				fileResolver.addManualClassPath(path);
			}
		}
		List<XmlNode> skipPatterns = nameFilter.findNodeList("skip-pattern");
		for (XmlNode skipPatter : skipPatterns) {
			String pattern = skipPatter.getAttribute("pattern");
			if (pattern != null && pattern.length() > 0) {
				logger.logMessage(LogLevel.INFO, "添加忽略文件正则表达式: [{0}]...",
						pattern);
				fileResolver.addSkipPathPattern(pattern);
			}
		}
	}

	private void setResolverOrder() {
		String order = fileResolverConfiguration.getAttribute("order");
		if (order != null && order.equalsIgnoreCase("true")) {
			fileResolver.setOrder(true);
		} else {
			fileResolver.setOrder(false);
		}
	}

	private void setResolverClassPath() {
		String resolverClassPath = fileResolverConfiguration
				.getAttribute("resolve-classpath");
		if (resolverClassPath != null
				&& resolverClassPath.equalsIgnoreCase("true")) {
			fileResolver.setClassPathResolve(true);
		} else {
			fileResolver.setClassPathResolve(false);
		}
	}

	private void configFileResolvers() {
		for (FileProcessor processor : fileResolver.getFileProcessorList()) {
			if (processor instanceof AppConfig) {
				AppConfigUtil.config(this, (AppConfig) processor);
			}
		}
	}

	public void stop() {
	}

	public String getNodeName() {
		return FILE_RESOLVER_NODE_NAME;
	}

	public XmlNode getConfiguration() {
		return fileResolverConfiguration;
	}

	private int getThreadNum(String threadNum) {
		int fileProcessorThreadNum = 1;
		if (threadNum != null) {
			try {
				fileProcessorThreadNum = Integer.parseInt(threadNum);
				if (fileProcessorThreadNum <= 0) {
					fileProcessorThreadNum = 1;
				}
			} catch (Exception e) {
				// 传入非int值
				fileProcessorThreadNum = 1;
			}

		}
		return fileProcessorThreadNum;

	}

}
